 SETLI 132
;**************************************************;
;            T E S T 1 6 K - T O 7                 ; 
;           (c) 2023 - Daniel Coulom               ;  
;           http://dcmoto.free.fr/                 ;
;--------------------------------------------------;
; Ce code est distribue gratuitement dans l'espoir ;
; qu'il sera utile, mais sans aucune  garantie  et ;
; sans  engager  la  responsabilit  de  l'auteur. ;
; Vous  pouvez  l' utiliser,  le  modifier  et  le ;
; diffuser librement, en conservant cette  licence ;
; et les rfrences de l'auteur dans   toutes  les ;
; copies. L'exploitation commerciale est interdite.;
;**************************************************;

; Ce programme teste l'extension memoire 16K du TO7.
; 1) Tester tous les bits (valeurs 0 et 1)
; 2) Ecrire des sequences de 255 octets diffrents 
; 3) Verifier que les memes sequences sont relues 

; L'extension mmoire est en $8000-$BFFF
; Le programme est charge et execute en $7000
; En version disquette le DOS ne permet pas le
; chargement direct en $7000. Le programme
; est charge en $9000. Il se translate lui-meme
; en $7000 et crase le DOS.


;**************************************************;
;                Version 2023.07.01                ;
;**************************************************;
; Historique
; 2023.07.01 Premiere version operationnelle

;---------------------------------
; Initialisations
;---------------------------------
  ORG   $7000
  LDS   #$7000         ; initialisation pile systeme                          
  ORCC  #$50           ; desactive les interruptions 

;---------------------------------
; Version disquette seulement :
; Le programme est charge en $9000
; puis translate de $9000 en $7000 
; Mettre en commentaire pour la
; version cassette.
;---------------------------------
;  LDX   #$9000
;  LDU   #$7000
;TRANS
;  LDD   ,X++
;  STD   ,U++
;  CMPX  #$9300
;  BNE   TRANS
;  JMP   DEBUT 

;---------------------------------
; Affichage du titre
;---------------------------------
DEBUT
  LDY    #TITRE        ; message de titre
  LBSR   WSTR          ; affichage message

;---------------------------------
; Test de chaque bit
;---------------------------------
TESTB0
  LDY    #TESTB        ; message "Test des bits"
  LBSR   WSTR          ; affichage message
TESTB1
  BSR    TBITS         ; test des bits de la banque
  BNE    ERBITS        ; erreur detectee
TESTB2
  LDY    #NOERR        ; message "Aucune erreur"
  LBSR   WSTR          ; affichage message
  BRA    TESTA0        ; test des adresses

;---------------------------------
; Erreur detectee sur les bits
;---------------------------------
ERBITS
  LDY    #ERR1         ; message "Erreur sur les bits"
  LBSR   WSTR          ; affichage message
  BRA    RETOUR        ; abandon du programme
  
;---------------------------------------
; Test de tous les bits
;---------------------------------------
TBITS
  LDX    #$8000        ; adresse de debut de l'extension
  LDB    #$55          ; valeur a ecrire en RAM
TBIT1
  COMB                 ; inversion des bits
  STB    ,X            ; ecriture
  CMPB   ,X            ; lecture
  BNE    TBIT2         ; valeur lue incorrecte
  COMB                 ; inversion des bits
  STB    ,X            ; ecriture
  CMPB   ,X            ; lecture
  BNE    TBIT2         ; valeur lue incorrecte
  CMPX   #$BFFF        ; adresse de fin de l'extension
  BEQ    TBIT2         ; fin de l'extension memoire
  LEAX   1,X           ; adresse suivante
  BRA    TBIT1         ; controle adresse suivante
TBIT2
  RTS       

;---------------------------------
; Test de toutes les adresses
;---------------------------------
TESTA0
  LDY    #WSEQU        ; message "Ecriture sequence"
  BSR    WSTR          ; affichage message
TESTA1
  BSR    WRITE         ; ecriture des sequences dans l'extension
TESTA2
  LDY    #RSEQU        ; message "Lecture sequence"
  BSR    WSTR          ; affichage message
TESTA3
  BSR    CTRL          ; controle des sequences dans l'extension
  BNE    ERADD         ; erreur detectee
TESTA4
  LDY    #NOERR        ; message "Aucune erreur"
  BSR    WSTR          ; affichage message
  BRA    RETOUR        ; retour         

;---------------------------------
; Erreur adresses
;---------------------------------
ERADD
  LDY    #ERR2         ; message "Erreur adresses"
  BSR    WSTR          ; affichage message

;---------------------------------
; Reinitialisation du TO7
;---------------------------------
RETOUR 
  LDY    #ADDRESS      ; message "  Adresse="
  BSR    WSTR          ; affichage message
  BSR    HEXVX         ; affichage valeur hexa de X
  LDY    #APPUI        ; message "Appuyez sur une touche pour sortir..."
  BSR    WSTR          ; affichage message
TOUCHE
  JSR    $E806         ; lecture clavier TO
  TSTB                 ; test de la touche
  BEQ    TOUCHE        ; attente d'une touche 
  JMP    [$FFFE]       ; hardreset

;------------------------------------------------------
; AFFICHAGE CHAINE TERMINEE PAR ZERO (Adresse dans Y)
;------------------------------------------------------
WSTR
  PSHS  X,B,A          ; sauvegarde registres
WSTR1  
  LDB   ,Y+            ; caractere a ecrire
  BEQ   WSTR9          ; fin de la chainee
  JSR   $E803          ; ecriture caractere
  BRA   WSTR1          ; caractere suivant
WSTR9
  PULS  A,B,X,PC       ; retour

;-----------------------------------------------
; Ecriture des sequences
;-----------------------------------------------
WRITE
  CLRB                 ; initialisation de la sequence
  LDX    #$8000        ; adresse de debut de l'extension
WRITE1
  INCB                 ; valeur suivante de $01 a $FE 
  CMPB   #$FF          ; fin de sequence
  BNE    WRITE2        ; ecrire la valeur
  CLRB                 ; recommencer a zero
WRITE2
  STB    ,X            ; ecriture
  CMPX   #$BFFF        ; adresse de fin de l'extension
  BEQ    WRITE3        ; fin de l'extension memoire
  LEAX   1,X           ; adresse suivante
  BRA    WRITE1        ; ecriture adresse suivante
WRITE3
  RTS

;-----------------------------------------------
; Controle des sequences
;-----------------------------------------------
CTRL
  CLRB                 ; initialisation de la sequence
  LDX    #$8000        ; adresse de debut de l'extension
CTRL1
  INCB                 ; valeur suivante de $01 a $FE 
  CMPB   #$FF          ; fin de sequence
  BNE    CTRL2         ; controler la valeur
  CLRB                 ; recommencer a zero
CTRL2
  CMPB   ,X            ; comparaison
  BNE    CTRL3         ; valeur lue incorrecte
  CMPX   #$BFFF        ; adresse de fin de l'extension
  BEQ    CTRL3         ; fin de l'extension memoire
  LEAX   1,X           ; adresse suivante
  BRA    CTRL1         ; controle adresse suivante
CTRL3
  RTS       

;------------------------------------------------------
; Affichage valeur hexadecimale
;------------------------------------------------------
DISPLAYHEXA
  LDY   #HEXA          ; valeur hexa
  BSR    WSTR          ; affichage valeur hexa
  RTS                  ; retour

;------------------------------------------------------
; CALCUL VALEUR HEXADECIMALE DE X
;------------------------------------------------------
HEXVX
  TFR   X,D            ; transfert dans D   
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  BSR   HCAR           ; calcul caractere
  STA   HEXA+1         ; stockage premier caractere
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  TFR   B,A            ; transfert dans A
  BSR   HCAR           ; calcul caractere
  STA   HEXA+3         ; stockage troisieme caractere
  TFR   X,D            ; transfert dans D   
  ANDA  #$0F           ; isole 4 bits poids faible   
  ANDB  #$0F           ; isole 4 bits poids faible   
  BSR   HCAR           ; calcul caractere
  STA   HEXA+2         ; stockage deuxieme caractere
  TFR   B,A            ; transfert dans A
  BSR   HCAR           ; calcul caractere
  STA   HEXA+4         ; stockage quatrieme caractere
  BRA   DISPLAYHEXA    ; affichage de la valeur hexadecimale

;------------------------------------------------------
; CALCUL VALEUR HEXADECIMALE DE A
;------------------------------------------------------
HEXVA
  PSHS  A              ; sauvegarde de A              
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  BSR   HCAR           ; calcul caractere
  STA   HEXA+1         ; stockage premier caractere
  PULS  A              ; restauration de A
  ANDA  #$0F           ; isole 4 bits poids faible   
  BSR   HCAR           ; calcul caractere
  STA   HEXA+2         ; stockage deuxieme caractere
  LDA   #$20           ; espace
  STA   HEXA+3         ; stockage troisieme caractere
  STA   HEXA+4         ; stockage quatrieme caractere
  BRA   DISPLAYHEXA    ; affichage de la valeur hexadecimale

;------------------------------------------------------
; CALCUL CARACTERE HEXADECIMAL (A)
;------------------------------------------------------
HCAR
  CMPA  #$0A           ; test lettre
  BLT   HCAR1          ; chiffre
  ADDA  #$07           ; ajout de 7 pour les lettres
HCAR1
  ADDA  #$30           ; ajout de $30 pour valeur ASCII
  RTS                  ; retour
          
;------------------------------------------------------
; Messages a afficher
;------------------------------------------------------
TITRE
  FCB   $0C            ; effacement ecran   
  FCB   $1B,$20,$40    ; couleur noir
  FCB   $1B,$20,$57    ; fond blanc
  FCB   $1B,$67        ; bordure blanche
  FCC   " -------------------------------"
  FCB   $0D,$0A
  FCC   "| Test extension memoire du TO7 |"
  FCB   $0D,$0A
  FCC   " -------------------------------"
  FCB   $0D,$0A,$0A
  FCB   $00
TESTB
  FCB   $0D,$0A,$0A
  FCC   "1) Test de chaque bit (0 et 1)"
  FCB   $0D,$0A,$00
WSEQU
  FCB   $0D,$0A,$0A
  FCC   "2) Ecriture sequence pour test adresses"
  FCB   $0D,$0A,$00
RSEQU
  FCB   $0D,$0A,$0A,$0A
  FCC   "3) Verification de la sequence ecrite"
  FCB   $0D,$0A,$00
ERR1  
  FCB   $0D,$0A,$0A
  FCC   "ERREUR DETECTEE SUR LES BITS"
  FCB   $0D,$0A,$00
ERR2  
  FCB   $0D,$0A
  FCC   "ERREUR DETECTEE SUR LES ADRESSES"
  FCB   $0D,$0A,$00
NOERR  
  FCB   $0D,$0A
  FCC   "AUCUNE ERREUR DETECTEE"
  FCB   $0D,$0A,$00
ADDRESS
  FCC   "Adresse="
  FCB   $00
APPUI
  FCB   $0D,$0A,$0A
  FCC   "Appuyez sur une touche pour sortir..."
  FCB   $00
HEXA
  FCB   $24,$30,$30,$30,$30,$00
